export install="/usr/bin/install -c"
export bash="/bin/bash"
export awk="/bin/gawk"
export grep="/bin/grep"
export make="/usr/bin/gmake"
export sed="/bin/sed"
export libtool="/opt/anaconda1anaconda2anaconda3/bin/libtool"
export libtoolize="/opt/anaconda1anaconda2anaconda3/bin/libtoolize"
export objcopy="/usr/bin/objcopy"
export objdump="/usr/bin/objdump"
export readelf="/usr/bin/readelf"
export patch="/usr/bin/patch"
export gperf="/usr/bin/gperf"
